/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wsf.container.jboss40;

// $Id: WebServiceClientDeployer.java 3414 2007-06-03 21:54:02Z thomas.diesler@jboss.com $

import java.util.Iterator;

import javax.naming.Context;

import org.jboss.deployment.DeploymentException;
import org.jboss.deployment.DeploymentInfo;
import org.jboss.system.ServiceMBeanSupport;
import org.jboss.util.naming.Util;
import org.jboss.webservice.metadata.serviceref.ServiceRefMetaData;
import org.jboss.ws.core.jaxrpc.client.ServiceReferenceable;
import org.jboss.ws.integration.URLLoaderAdapter;
import org.jboss.wsf.container.jboss42.DeploymentInfoAdapter;
import org.jboss.wsf.container.jboss42.DeploymentInfoAdapterFactory;
import org.jboss.wsf.spi.deployment.BasicDeployment;
import org.jboss.wsf.spi.deployment.Deployment;
import org.jboss.wsf.spi.deployment.JAXRPCClientDeployment;
import org.jboss.wsf.spi.deployment.UnifiedDeploymentInfo;
import org.jboss.wsf.spi.metadata.j2ee.serviceref.UnifiedServiceRefMetaData;

/**
 * Binds a JAXRPC Service object in the client's ENC for every service-ref element in the
 * deployment descriptor.
 *
 * @author Thomas.Diesler@jboss.org
 * @since 15-Jan-2005
 */
public class WebServiceClientDeployer extends ServiceMBeanSupport implements WebServiceClientDeployerMBean
{
   /**
    * This binds a jaxrpc Service into the callers ENC for every service-ref element
    *
    * @param envCtx      ENC to bind the javax.rpc.xml.Service object to
    * @param serviceRefs An iterator of the service-ref elements in the client deployment descriptor
    * @param di          The client's deployment info
    * @throws org.jboss.deployment.DeploymentException if it goes wrong
    */
   public void setupServiceRefEnvironment(Context envCtx, Iterator serviceRefs, DeploymentInfo di) throws DeploymentException
   {
      try
      {
         while (serviceRefs.hasNext())
         {
            ServiceRefMetaData serviceRef = (ServiceRefMetaData)serviceRefs.next();
            String serviceRefName = serviceRef.getServiceRefName();

            // Build the container independent deployment info
            Deployment dep = new BasicDeployment();
            UnifiedDeploymentInfo udi = new JAXRPCClientDeployment(Deployment.DeploymentType.JAXRPC_CLIENT);
            DeploymentInfoAdapter adapter = DeploymentInfoAdapterFactory.getDeploymentInfoAdapter();
            adapter.buildDeploymentInfo(dep, udi, di);
            
            UnifiedServiceRefMetaData usr = ServiceRefMetaDataAdapterFactory.newInstance().buildServiceRefMetaData(udi, serviceRef);
            usr.setVfsRoot(new URLLoaderAdapter(udi.url));

            ServiceReferenceable ref = new ServiceReferenceable(usr);
            Util.bind(envCtx, serviceRefName, ref);

            log.debug("Webservice binding: java:comp/env/" + serviceRefName);
         }
      }
      catch (Exception e)
      {
         throw new DeploymentException("Cannot bind webservice to client environment", e);
      }
   }
}
